# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Wanha::Nick;

use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::DB;


my %name_to_id;
my %id_to_name;


=item add
Add nick to database

0: nick name
r: new nick id
=cut
sub add($) {
	my ($name) = @_;
	my $id;
	my $update;

	&Wanha::DB::transaction_start();
	$id = &Wanha::DB::get_sequence("nicks_id_seq");
	$update = &Wanha::DB::prepare(
			"INSERT INTO nicks (id, nick) VALUES (&da&, &sb&)");
	$update = &Wanha::DB::set($update,
			"a", $id,
			"b", $name);
	&Wanha::DB::update($update);
	&Wanha::DB::transaction_end();

	$name_to_id{$name} = $id;
	$id_to_name{$id} = $name;

	return $id;
}


=item get_id
Get nick ID (by name)

0: nick name
1: audo add
r: nick id
=cut
sub get_id($$) {
	my ($name, $autoadd) = @_;
	my ($query, $result);

	(defined($name_to_id{$name})) && return $name_to_id{$name};

	$query = &Wanha::DB::prepare(
			"SELECT id, nick FROM nicks WHERE nick ILIKE &sa&");
	$query = &Wanha::DB::set($query, "a", $name);
	$result = &Wanha::DB::query($query);
	if (&Wanha::DB::row_count($result) == 1) {
		my ($id, $t) = &Wanha::DB::get_row($result);
		$name_to_id{$name} = $id;
		return $id;
	} elsif (&Wanha::DB::row_count($result) == 0) {
		if (defined($autoadd) && $autoadd == 1) {
			return &add($name);
		}
		return undef;
	}

	# more than one entry? PostgreSQL/ILIKE seems to handle '_' as a
	# wildcard... Fine, we'll manually pick the correct one.
	$name =~ tr/A-Z/a-z/;
	while (my ($id, $str) = &Wanha::DB::get_row($result)) {
		$str =~ tr/A-Z/a-z/;
		if ($str eq $name) {
			$name_to_id{$name} = $id;
			return $id;
		}
	}
	return undef;
}


=item get_name
Get nick name (by ID)

0: id
r: nick name
=cut
sub get_name($) {
	my ($id) = @_;
	my ($query, $result);

	(defined($id_to_name{$id})) && return $id_to_name{$id};

	$query = &Wanha::DB::prepare("SELECT nick FROM nicks WHERE id = &da&");
	$query = &Wanha::DB::set($query, "a", $id);
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return undef;

	my ($name) = &Wanha::DB::get_row($result);
	$id_to_name{$id} = $name;

	return $name;
}


=item get_names
Get several nick names (by ID)

@: nick id
r: hash id->nick
=cut
sub get_names(@) {
	my @ids = @_;
	my ($query, $result);

	my @prep_ids = split(//, "abcdefghijklmnopqrstuvwxyz" .
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ");

	$query = "SELECT id, nick FROM nicks WHERE ";
	for (my $i = 0; $i <= $#ids; $i++) {
		$query .= sprintf("id = &d%s& OR ", $prep_ids[$i]);
	}
	$query =~ s/OR $//;
	$query = &Wanha::DB::prepare($query);

	for (my $i = 0; $i <= $#ids; $i++) {
		$query = &Wanha::DB::set($query, $prep_ids[$i], $ids[$i]);
	}

	$result = &Wanha::DB::query($query);
	my %names;
	while (my ($id, $name) = &Wanha::DB::get_row($result)) {
		$names{$id} = $name;
	}

	return %names;
}


=item set_mute
Set nick mute

0: nick id
1: mute (0 = false, 1 = true)
=cut
sub set_mute($$) {
	my ($id, $mute) = @_;
	my $update;

	$update = &Wanha::DB::prepare(
			"UPDATE nicks SET mute = &ba& WHERE id = &db&");
	$update = &Wanha::DB::set($update,
			"a", $mute,
			"b", $id);
	&Wanha::DB::update($update);
}


=item get_mute
Get nick mute

0: nick id
r: 0 = false, 1 = true
=cut
sub get_mute($) {
	my ($id) = @_;
	my ($query, $result);

	$query = &Wanha::DB::prepare("SELECT mute FROM nicks WHERE id = &da&");
	$query = &Wanha::DB::set($query, "a", $id);
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 1) || die "user $id does not exist!";

	my ($mute) = &Wanha::DB::get_row($result);
	$mute = &Wanha::DB::bool_to_int($mute);
	return $mute;
}


return 1;
